// A door trigger actor that will open a door when living actors are nearby.
class DoorTrigger : Actor {
  Default {
    //$Title "Door Trigger"
    //$Category "Door"

    //$Arg0 "Tag 1"
    //$Arg0Type 13

    //$Arg1 "Tag 2"
    //$Arg1Type 13

    //$Arg2 "Radius"
    //$Arg2RenderStyle "circle"

    //$IgnoreRenderstyle

    //$Sprite edtoa0

    Radius 16;
    Height 16;
    Mass 1;

    +NOBLOCKMAP
    +NOGRAVITY
    +NOSECTOR
    +NOINTERACTION
  }

  States {
    Spawn:
      TNT1 A -1;
      Loop;
  }

  Actor helperA;
  Actor helperB;

  bool isOpen;
  Sector sectorA;
  Sector sectorB;
  int delay;

  const SECTOR_SPEED = 64;
  const CHECK_INTERVAL = 5;

  override void BeginPlay() {
    super.BeginPlay();

    isOpen = false;

    SectorTagIterator sectorsA = SectorTagIterator.Create(args[0]);
    sectorA = level.sectors[sectorsA.Next()];

    SectorTagIterator sectorsB = SectorTagIterator.Create(args[1]);
    sectorB = level.sectors[sectorsB.Next()];

    // Randomize first full update so that not all door trigger actors are
    // fully updating every tick.
    delay = random(0, CHECK_INTERVAL);

    helperA = Spawn("MapSpot", Vec3Angle(48, angle));
    helperB = Spawn("MapSpot", Vec3Angle(48, angle + 180.0));
  }

  override void Tick() {
    super.Tick();

    // Do not execute every tick for better performance.
    if (--delay > 0) {
      return;
    }
    delay = CHECK_INTERVAL;

    // Wait with full update until the door sectors have stopped moving.
    if (sectorA.PlaneMoving(0) || sectorA.PlaneMoving(1) ||
        sectorB.PlaneMoving(0) || sectorB.PlaneMoving(1)) {
      return;
    }

    int distance = args[2];

    // Find out if any valid actors are near. Valid actors are shootable and solid, so that
    // only players or monsters that are alive are counted.
    bool actorsNear = false;
    BlockThingsIterator it = BlockThingsIterator.Create(self, distance);
    while (it.Next()) {
      let targ = it.thing;
      if (targ != null) {

        // Exclude things that are not "alive".
        if (!targ.bShootable || !targ.bSolid) {
          continue;
        }

        // Exclude monsters not yet awake.
        if (!targ.Player && !targ.Target) {
          continue;
        }

        // Exclude targets that are too far away.
        if (Distance3D(targ) > distance) {
          continue;
        }

        // Must be visible from one of the helper actors.
        if (!helperA.IsVisible(targ, true) && !helperB.IsVisible(targ, true)) {
          continue;
        }

        actorsNear = true;
        break;
      }
    }

    // Toggle door state based on nearby actors and current state.
    int tagA = args[0];
    int tagB = args[1];
    if (!isOpen && actorsNear) {
      isOpen = true;
      A_DoorOpen(tagA, tagB);

    } else if (isOpen && !actorsNear) {
      isOpen = false;
      A_DoorClose(tagA, tagB);

    }
  }

  void A_DoorOpen(int tagA, int tagB) {
    StopSoundSequence();
    StartSoundSequence("AutoDoorOpen", 0);

    Generic_Floor(tagA, SECTOR_SPEED, 80, 0, 16);
    Generic_Floor(tagB, SECTOR_SPEED, 48, 0, 16);
    Generic_Ceiling(tagA, SECTOR_SPEED, 72, 0, 16 + 8);
    Generic_Ceiling(tagB, SECTOR_SPEED, 72, 0, 16 + 8);
  }

  void A_DoorClose(int tagA, int tagB) {
    StopSoundSequence();
    StartSoundSequence("AutoDoorClose", 0);

    Generic_Floor(tagA, SECTOR_SPEED, 80, 0, 16 + 8);
    Generic_Floor(tagB, SECTOR_SPEED, 48, 0, 16 + 8);
    Generic_Ceiling(tagA, SECTOR_SPEED, 72, 0, 16);
    Generic_Ceiling(tagB, SECTOR_SPEED, 72, 0, 16);
  }
}
